"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const ts = __importStar(require("typescript"));
const pluralize_1 = __importDefault(require("pluralize"));
const lint_1 = require("../lint");
const Builder_1 = require("./Builder");
const utils_1 = require("./utils");
const foreignKeyDecorator = 'ForeignKey';
/**
 * @class ModelGenerator
 * @constructor
 * @param {Dialect} dialect
 */
class ModelBuilder extends Builder_1.Builder {
    constructor(config, dialect) {
        super(config, dialect);
    }
    /**
     * Build column class member
     * @param {IColumnMetadata} col
     * @param {Dialect} dialect
     */
    static buildColumnPropertyDecl(col, dialect) {
        var _a;
        const buildColumnDecoratorProps = (col) => {
            const props = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, col.originName && col.name !== col.originName && { field: col.originName }), col.primaryKey && { primaryKey: col.primaryKey }), col.autoIncrement && { autoIncrement: col.autoIncrement }), col.allowNull && { allowNull: col.allowNull }), col.dataType && { type: col.dataType }), col.comment && { comment: col.comment });
            return props;
        };
        const buildIndexDecoratorProps = (index) => {
            const props = Object.assign(Object.assign(Object.assign({ name: index.name }, index.using && { using: index.using }), index.collation && { order: index.collation === 'A' ? 'ASC' : 'DESC' }), { unique: index.unique });
            return props;
        };
        return ts.createProperty([
            ...(col.foreignKey ?
                [utils_1.generateArrowDecorator(foreignKeyDecorator, [col.foreignKey.targetModel])]
                : []),
            utils_1.generateObjectLiteralDecorator('Column', buildColumnDecoratorProps(col)),
            ...(col.indices && col.indices.length ?
                col.indices.map(index => utils_1.generateObjectLiteralDecorator('Index', buildIndexDecoratorProps(index)))
                : [])
        ], undefined, col.name, (col.autoIncrement || col.allowNull) ?
            ts.createToken(ts.SyntaxKind.QuestionToken) : ts.createToken(ts.SyntaxKind.ExclamationToken), ts.createTypeReferenceNode((_a = dialect.mapDbTypeToJs(col.type), (_a !== null && _a !== void 0 ? _a : 'any')), undefined), undefined);
    }
    /**
     * Build association class member
     * @param {IAssociationMetadata} association
     */
    static buildAssociationPropertyDecl(association) {
        const { associationName, targetModel, joinModel } = association;
        const targetModels = [targetModel];
        joinModel && targetModels.push(joinModel);
        return ts.createProperty([
            ...(association.sourceKey ?
                [
                    utils_1.generateArrowDecorator(associationName, targetModels, { sourceKey: association.sourceKey })
                ]
                : [
                    utils_1.generateArrowDecorator(associationName, targetModels)
                ]),
        ], undefined, associationName.includes('Many') ?
            pluralize_1.default.plural(targetModel) : pluralize_1.default.singular(targetModel), ts.createToken(ts.SyntaxKind.QuestionToken), associationName.includes('Many') ?
            ts.createArrayTypeNode(ts.createTypeReferenceNode(targetModel, undefined)) :
            ts.createTypeReferenceNode(targetModel, undefined), undefined);
    }
    /**
     * Build table class declaration
     * @param {ITableMetadata} tableMetadata
     * @param {Dialect} dialect
     */
    static buildTableClassDeclaration(tableMetadata, dialect) {
        var _a, _b;
        const { originName: tableName, name, columns } = tableMetadata;
        const classDecl = ts.createClassDeclaration([
            // @Table decorator
            utils_1.generateObjectLiteralDecorator('Table', Object.assign(Object.assign(Object.assign({ tableName: tableName }, tableMetadata.schema && { schema: tableMetadata.schema }), { timestamps: tableMetadata.timestamps }), tableMetadata.comment && { comment: tableMetadata.comment }))
        ], [
            ts.createToken(ts.SyntaxKind.ExportKeyword),
        ], name, undefined, [
            ts.createHeritageClause(ts.SyntaxKind.ExtendsKeyword, [
                ts.createExpressionWithTypeArguments([ts.createTypeReferenceNode(name, undefined)], ts.createIdentifier('Model'))
            ])
        ], 
        // Class members
        [
            ...Object.values(columns).map(col => this.buildColumnPropertyDecl(col, dialect)),
            ...tableMetadata.associations && tableMetadata.associations.length ?
                tableMetadata.associations.map(a => this.buildAssociationPropertyDecl(a)) : []
        ]);
        let generatedCode = '';
        // Named imports from sequelize-typescript
        generatedCode += utils_1.nodeToString(utils_1.generateNamedImports([
            'Model',
            'Table',
            'Column',
            'DataType',
            'Index',
            foreignKeyDecorator,
            ...new Set((_a = tableMetadata.associations) === null || _a === void 0 ? void 0 : _a.map(a => a.associationName)),
        ], 'sequelize-typescript'));
        generatedCode += '\n';
        // Named imports for associations
        const importModels = new Set();
        // Add models for associations
        (_b = tableMetadata.associations) === null || _b === void 0 ? void 0 : _b.forEach(a => {
            importModels.add(a.targetModel);
            a.joinModel && importModels.add(a.joinModel);
        });
        // Add models for foreign keys
        Object.values(tableMetadata.columns).forEach(col => {
            col.foreignKey && importModels.add(col.foreignKey.targetModel);
        });
        [...importModels].forEach(modelName => {
            generatedCode += utils_1.nodeToString(utils_1.generateNamedImports([modelName], `./${modelName}`));
            generatedCode += '\n';
        });
        generatedCode += utils_1.nodeToString(classDecl);
        return generatedCode;
    }
    /**
     * Build main index file
     * @param {ITableMetadata[]} tablesMetadata
     * @returns {string}
     */
    static buildIndexExports(tablesMetadata) {
        return Object.values(tablesMetadata)
            .map(t => utils_1.nodeToString(utils_1.generateIndexExport(t.name)))
            .join('\n');
    }
    /**
     * Build models files using the given configuration and dialect
     * @returns {Promise<void>}
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            const { clean, outDir } = this.config.output;
            const writePromises = [];
            const tablesMetadata = yield this.dialect.buildTablesMetadata(this.config);
            if (Object.keys(tablesMetadata).length === 0) {
                console.warn(`Couldn't find any table for database ${this.config.connection.database} and provided filters`);
                return;
            }
            // Check if output dir exists
            try {
                yield fs_1.promises.access(outDir);
            }
            catch (err) {
                if (err.code && err.code === 'ENOENT') {
                    yield fs_1.promises.mkdir(outDir, { recursive: true });
                }
                else {
                    console.error(err);
                    process.exit(1);
                }
            }
            // Clean files if required
            if (clean) {
                for (const file of yield fs_1.promises.readdir(outDir)) {
                    yield fs_1.promises.unlink(path_1.default.join(outDir, file));
                }
            }
            // Build model files
            for (const tableMetadata of Object.values(tablesMetadata)) {
                const tableClassDecl = ModelBuilder.buildTableClassDeclaration(tableMetadata, this.dialect);
                writePromises.push((() => __awaiter(this, void 0, void 0, function* () {
                    const outPath = path_1.default.join(outDir, `${tableMetadata.name}.ts`);
                    yield fs_1.promises.writeFile(outPath, tableClassDecl, { flag: 'w' });
                    console.log(`Generated file ${outPath}`);
                }))());
            }
            // Build index file
            writePromises.push((() => __awaiter(this, void 0, void 0, function* () {
                const indexPath = path_1.default.join(outDir, 'index.ts');
                const indexContent = ModelBuilder.buildIndexExports(tablesMetadata);
                yield fs_1.promises.writeFile(indexPath, indexContent);
                console.log(`Generated file ${indexPath}`);
            }))());
            yield Promise.all(writePromises);
            // Lint files
            let linter;
            if (this.config.lintOptions) {
                linter = new lint_1.Linter(this.config.lintOptions);
            }
            else {
                linter = new lint_1.Linter();
            }
            linter.lintFiles([path_1.default.join(outDir, '*.ts')]);
        });
    }
}
exports.ModelBuilder = ModelBuilder;
