"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const printer = ts.createPrinter({ newLine: ts.NewLineKind.LineFeed });
/**
 * Returns string representation of typescript node
 * @param node
 * @returns {string}
 */
exports.nodeToString = (node) => {
    const sourceFile = ts.createSourceFile(`source.ts`, ``, ts.ScriptTarget.Latest, false, ts.ScriptKind.TS);
    return printer.printNode(ts.EmitHint.Unspecified, node, sourceFile);
};
/**
 * Generate named imports code (e.g. `import { Something, Else } from "module"`)
 * @param {string[]} importsSpecifier
 * @param {string} moduleSpecifier
 * @returns {string} Named import code
 */
exports.generateNamedImports = (importsSpecifier, moduleSpecifier) => {
    return ts.createImportDeclaration(
    /* decorators */ undefined, 
    /* modifiers */ undefined, ts.createImportClause(undefined, ts.createNamedImports([
        ...importsSpecifier
            .map(is => ts.createImportSpecifier(undefined, ts.createIdentifier(is)))
    ])), ts.createLiteral(moduleSpecifier));
};
/**
 * Generate model export for index file
 * @param {string} modelFileName
 * @returns {ts.ExportDeclaration}
 */
exports.generateIndexExport = (modelFileName) => {
    return ts.createExportDeclaration([], undefined, undefined, ts.createLiteral(`./${modelFileName}`));
};
/**
 * Generate object literal decorator
 * @param {string} decoratorIdentifier
 * @param {[key: string]: any} props
 * @return {ts.Decorator}
 */
exports.generateObjectLiteralDecorator = (decoratorIdentifier, props) => {
    return ts.createDecorator(ts.createCall(ts.createIdentifier(decoratorIdentifier), undefined, [
        ts.createObjectLiteral([
            ...Object.entries(props)
                .map(e => ts.createPropertyAssignment(e[0], typeof e[1] === 'string' && e[1].startsWith('DataType.') ?
                ts.createIdentifier(e[1]) : ts.createLiteral(e[1])))
        ])
    ]));
};
/**
 * Generate arrow decorator
 * @param {string} decoratorIdentifier
 * @param {string[]} arrowTargetIdentifiers
 * @param {object} objectLiteralProps
 * @returns {ts.Decorator}
 */
exports.generateArrowDecorator = (decoratorIdentifier, arrowTargetIdentifiers, objectLiteralProps) => {
    const argumentsArray = arrowTargetIdentifiers.map(t => ts.createArrowFunction(undefined, undefined, [], undefined, ts.createToken(ts.SyntaxKind.EqualsGreaterThanToken), ts.createIdentifier(t)));
    objectLiteralProps && argumentsArray.push(ts.createObjectLiteral([
        ...Object.entries(objectLiteralProps).map(e => ts.createPropertyAssignment(e[0], ts.createLiteral(e[1])))
    ]));
    return ts.createDecorator(ts.createCall(ts.createIdentifier(decoratorIdentifier), undefined, argumentsArray));
};
